/*
 * @(#)ses_event.h 1.13 91/01/11 Copyr 1986 Sun Micro
 */

/*
 * Data size restrictions
 */
#define SES_MAX_N_UDATA	512	/* maximum user data size:
				 * 	connect 
				 *	release			
				 */
#define SES_MAX_UABORT_UDATA 9	/* maximum user data size:
				 *	user abort
				 */
#define SES_MAX_EX_DATA	14	/* maximum user data size:
				 *	 expedited data
				 */
#define SES_MAX_IDUSIZE	8000	/* maximum interface data unit size */

#define SES_MAX_RFL_DATA 9	/* maximum reflect paramter size */




/*
 * Event parameter types
 */
#define SES_REF_SIZE	64	/* maximum possible reference id size */
typedef struct {		/* connection reference identification */
	unsigned size;			/* id size */
	u_char id[SES_REF_SIZE];		/* id number */
} SES_REF_ID;

#define SES_ADD_REF_SIZE 4
typedef struct {
	unsigned	size;			/* size of data */
	u_char		info[SES_ADD_REF_SIZE]; /* data 	*/
} SES_ADD_REF;

typedef struct {		/* quality of service */
	unsigned	dummy;		/* body to be provided */
} SES_QOS;

typedef unsigned SES_FU_DOMAIN;	/* set of optional fu's defined in protocol */
#define	SES_FUD_HD	0x001 		/* half-duplex fu */
#define	SES_FUD_FD	0x002 		/* +duplex fu */
#define	SES_FUD_EX	0x004 		/* expedited data fu */
#define	SES_FUD_SY	0x008 		/* minor synchronize fu */
#define	SES_FUD_MA	0x010 		/* major synchronize fu */
#define	SES_FUD_RESYN	0x020		/* resynchronize fu */
#define	SES_FUD_ACT	0x040 		/* activity management fu */
#define	SES_FUD_NR	0x080 		/* negotiated release fu */
#define	SES_FUD_CD	0x100		/* capability data fu */
#define	SES_FUD_EXCPT	0x200 		/* exceptions fu */
#define	SES_FUD_TD	0x400 		/* typed data fu */

typedef u_char SES_TK_DOMAIN;	/* set of tokens specifed in protocol */
#define	SES_TKD_MI	0x04		/* synchronize-minor token (PV bit 3)*/
#define	SES_TKD_MA	0x10		/* synchronize-major token (PV bit 5)*/
#define	SES_TKD_TR	0x40		/* release token (PV bit 7) */
#define	SES_TKD_DK	0x01		/* data token (PV bit 1) */
#define SES_TKD_ALL	(SES_TKD_MI | SES_TKD_MA | SES_TKD_TR | SES_TKD_DK)
#define SES_TKD_NONE	(SES_TK_DOMAIN)0

#ifdef del
typedef enum {			/* types of tokens specifed in protocol */
	SES_TKT_MI	= 0,		/* synchronize-minor token */
	SES_TKT_MA	= 1,		/* synchronize-major token */
	SES_TKT_TR	= 2,		/* release token */
	SES_TKT_DK	= 3		/* data token */
} SES_TK_TYPE;
#endif del

typedef enum {			/* token assignment encoding */
	SES_TAE_INIT = 0,		/* initiator's side */
	SES_TAE_RESP = 1,		/* responder's side */
	SES_TAE_CD_UCH = 2,		/* called user's choice */
	SES_TAE_RESV3 = 3,		/* reserved */
	SES_TAE_UNUSED = 4		/* unused or unavailable */
} SES_TA_ENCODE;

typedef struct { 		/* token assignment */
	SES_TA_ENCODE rel_tk:4;		/* release token */
	SES_TA_ENCODE act_tk:4;		/* major/activity token */
	SES_TA_ENCODE sync_tk:4;	/* minor/synchronize token */
	SES_TA_ENCODE data_tk:4;	/* data token */
} SES_TOK_ASSIGN;


typedef enum { 			/* connection request result */
	SES_CRR_ACCEPT = 0,		/* connection accepted */
	SES_CRR_U_UNSPEC = 1,		/* rejected by SS-user, reason 
					   unspecified */
	SES_CRR_U_BUSY = 2,		/* rejected by SS-user, temporary 
					   congestion */
	SES_CRR_U_USRSPEC = 3,		/* rejected by SS-user, user data field
					   may be used to provide further 
					   information */
	SES_CRR_P_UNSPEC = 4,		/* rejected by SS-provider,  reason 
					   unspecified */
	SES_CRR_P_BUSY = 5,		/* rejected by SS-provider, temporary 
					   congestion */
	SES_CRR_P_UKNSSAP = 6,		/* rejected by SS-provider, called 
					   SSAP unknown */
	SES_CRR_P_NOUSR = 7		/* rejected by SS-provider, called 
					   SS-user not attached to SSAP */
} SES_CR_RESULT;

typedef enum { 			/* SS-provider abort reason */
	SES_PAR_TRDISC = 0,		/* transport disconnect */
	SES_PAR_PROTO = 1,		/* protocol error */
	SES_PAR_UNSPEC = 2,		/* unspecified */
} SES_PA_REASON;

typedef enum {			/* SS-provider exception report reason */
	SES_PERR_PROTO,			/* protocol error */
	SES_PERR_UNSPEC			/* unspecified */
} SES_PER_REASON;

typedef enum {		/* Reason for UER, ACTI and ACTD */
	SES_UERR_U_RCV_ERR = 1,		/* User receiving ability jeopardized*/
	SES_UERR_U_LOCAL = 5,		/* local user error		*/
	SES_UERR_SEQ_ERR = 3,		/* sequence error		*/
	SES_UERR_DEMAND_DT = 128,	/* demand data token		*/
	SES_UERR_PROC_ERR = 6,		/* unrecoverable procedure error*/
	SES_UERR_NON_SPEC = 0		/* non-specific error		*/
} SES_UER_REASON;

typedef enum {			/* SS-user protocol error */
	SES_UPER_BADFU,			/* invalid fu's requested */
	SES_UPER_BADTSAP,		/* wrong tsap for existing connection*/
	SES_UPER_BADSSAP,		/* invalid SSAP specified for this 
					 * implemenation of session */
	SES_UPER_BADUCNREF,		/* Invalid user connection reference,
					 * user_cnref.size > SES_REF_SIZE */
	SES_UPER_BADCCNREF,		/* Invalid common connection reference,
					 * comm_cnref.size > SES_REF_SIZE */
	SES_UPER_BADADREF		/* Invalid additional reference,
					 * add_ref.size > SES_ADD_REF_SIZE or 
					 * add_ref.size < 0 */
} SES_UPE_REASON;

#define SES_SPSN_SIZE 6
typedef unsigned SES_SPSN;	/* Sync point serial number (0-999998) */
				/* if this is changed, see CTL_OUT_SPSN in
				 * osi_layer.h */

typedef enum {			/* Resynchronization type */
	SES_RSYNC_ABANDON,
	SES_RSYNC_RESTART,
	SES_RSYNC_SET
} SES_RSYNC_TYPE;

#define SES_ACT_ID_SIZE 6
typedef struct {		/* Activity Identifier */
	short	size;
	u_char	data[SES_ACT_ID_SIZE];
} SES_ACT_ID;



/*
 * Event identifiers
 */
#define SES_UEVENT	100
#define SES_PEVENT	200
typedef enum {
	SES_INVALID = 0,
				/* SS-user generated event */
	SES_CONNECT_REQ = SES_UEVENT,	/* S-CONNECT request */
	SES_CONNECT_RSP,		/* S-CONNECT response  */
	SES_DATA_REQ,			/* S-DATA request */
	SES_XDATA_REQ,			/* S-EXPEDITED-DATA request */
	SES_TDATA_REQ,			/* S-TYPED-DATA request */
	SES_CDATA_REQ,			/* S-CAPABILITY-DATA request */
	SES_CDATA_RSP,			/* S-CAPABILITY-DATA response */
	SES_TOK_GV_REQ,			/* S-TOKEN-GIVE request */
	SES_TOK_PL_REQ,			/* S-TOKEN-PLEASE request */
	SES_CNTL_GV_REQ,		/* S-CONTROL-GIVE request */
	SES_SYNC_MI_REQ,		/* S-SYNC-MINOR request */
	SES_SYNC_MI_RSP,		/* S-SYNC-MINOR response */
	SES_SYNC_MA_REQ,		/* S-SYNC-MAJOR request */
	SES_SYNC_MA_RSP,		/* S-SYNC-MAJOR response */
	SES_RESYNC_REQ,			/* S-RESYNCHRONIZE request */
	SES_RESYNC_RSP,			/* S-RESYNCHRONIZE response */
	SES_UER_REQ,			/* S-U-EXCEPTION-REPORT request */
	SES_ACTS_REQ,			/* S-ACTIVITY-START request */
	SES_ACTR_REQ,			/* S-ACTIVITY-RESUME request */
	SES_ACTI_REQ,			/* S-ACTIVITY-INTERRUPT request */
	SES_ACTI_RSP,			/* S-ACTIVITY-INTERRUPT response */
	SES_ACTD_REQ,			/* S-ACTIVITY-DISCARD request */
	SES_ACTD_RSP,			/* S-ACTIVITY-DISCARD response */
	SES_ACTE_REQ,			/* S-ACTIVITY-END request */
	SES_ACTE_RSP,			/* S-ACTIVITY-END response */
	SES_RELEASE_REQ,		/* S-RELEASE request */
	SES_RELEASE_RSP,		/* S-RELEASE response  */
	SES_UABORT_REQ, 		/* S-U-ABORT request */
#define SES_MAX_UEVENT (unsigned)SES_UABORT_REQ

				/* SS-provider generated event */
	SES_CONNECT_CNF = SES_PEVENT,	/* S_CONNECT confirm */
	SES_CONNECT_IND,		/* S-CONNECT indication */
	SES_DATA_IND,			/* S-DATA indication */
	SES_XDATA_IND,			/* S-XDATA indication */
	SES_TDATA_IND,			/* S-TYPED-DATA indication */
	SES_CDATA_IND,			/* S-CAPABILITY-DATA indication */
	SES_CDATA_CNF,			/* S-CAPABILITY-DATA confirmation */
	SES_TOK_GV_IND,			/* S-TOKEN-GIVE indication */
	SES_TOK_PL_IND,			/* S-TOKEN-PLEASE indication */
	SES_CNTL_GV_IND,		/* S-CONTROL-GIVE indication */
	SES_SYNC_MI_IND,		/* S-SYNC-MINOR indication */
	SES_SYNC_MI_CNF,		/* S-SYNC-MINOR confirmation */
	SES_SYNC_MA_IND,		/* S-SYNC-MAJOR indication */
	SES_SYNC_MA_CNF,		/* S-SYNC-MAJOR confirmation */
	SES_RESYNC_IND,			/* S-RESYNCHRONIZE indication */
	SES_RESYNC_CNF,			/* S-RESYNCHRONIZE confirmation */
	SES_PER_IND,			/* S-P-EXCEPTION-REPORT indication */
	SES_UER_IND,			/* S-U-EXCEPTION-REPORT indication */
	SES_ACTS_IND,			/* S-ACTIVITY-START indication */
	SES_ACTR_IND,			/* S-ACTIVITY-RESUME indication */
	SES_ACTI_IND,			/* S-ACTIVITY-INTERRUPT indication */
	SES_ACTI_CNF,			/* S-ACTIVITY-INTERRUPT confirmation */
	SES_ACTD_IND,			/* S-ACTIVITY-DISCARD indication */
	SES_ACTD_CNF,			/* S-ACTIVITY-DISCARD confirmation */
	SES_ACTE_IND,			/* S-ACTIVITY-END indication */
	SES_ACTE_CNF,			/* S-ACTIVITY-END confirmation */
	SES_RELEASE_CNF,		/* S-RELEASE confirm */
	SES_RELEASE_IND,		/* S-RELEASE indication */
	SES_UABORT_IND,			/* S-U-ABORT indication */
	SES_PABORT_IND,			/* S-P-ABORT indication */
	SES_UPROTO,			/* Protocol violation by user-generated
					   event */
	SES_PPROTO			/* Protocol violation by peer-generated
					   event */
#define SES_MAX_PEVENT (unsigned)SES_PPROTO
} SES_EVENT;

/*
 *  A "funky" event is one which causes the FSM to become out of date
 *  (see the comments near EV_FSM_OK in ses_action.c).
 */
#define FUNKY_EVENT(event) \
	( (event == SES_SYNC_MI_REQ) \
	    || (event == SES_SYNC_MA_REQ) \
	    || (event == SES_ACTE_REQ) )


/*
 * Event message definitions
 */

typedef struct {	/* generic message format */
	SES_EVENT	event;		/* message type */
} SES_MSG;


typedef struct {	/* connect service message format */
	SES_EVENT	event;		/* message type */
	SES_REF_ID	user_cnref;	/* called-user/calling-user 
					 * connection reference */
	SES_REF_ID	comm_cnref;	/* common connection reference */
	SES_ADD_REF	add_ref;	/* additional reference info */
	OSI_ADDR	cg_ssap;	/* calling SSAP */
	OSI_ADDR	cd_ssap;	/* called SSAP */
	SES_QOS		qos;		/* quality of service */
	SES_FU_DOMAIN	su_req;		/* session user requirements */
	SES_TOK_ASSIGN	tk_assign;	/* initial token assignment */
	SES_SPSN	init_spsn;	/* initial sync point serial number */
	SES_CR_RESULT	cr;		/* connection-request result */
	PROFILE_VECTOR_ARRAY	pv_all;	/* profile vector array		*/
} SES_MSG_CONNECT;


typedef struct {	/* data transfer service message format */
	SES_EVENT	event;		/* message type */
	unsigned	eosdu;		/* end of SSDU */
} SES_MSG_DATA;


typedef struct {	/* expedited data transfer service message format */
	SES_EVENT	event;		/* message type */
} SES_MSG_XDATA;

typedef SES_MSG_DATA 	/* typed data transfer service message format */
	SES_MSG_TDATA;

#ifdef del
typedef struct {	
	SES_EVENT	event;		/* message type */
	unsigned	eosdu;		/* end of SSDU */
} SES_MSG_TDATA;
#endif

typedef struct {	/* capability data transfer service message format */
	SES_EVENT	event;		/* message type */
} SES_MSG_CDATA;


typedef struct {	/* session give token message format */
	SES_EVENT	event;		/* message type */
	SES_TK_DOMAIN	tk_give;	/* tokens given */
} SES_MSG_TOKGV;

typedef struct {	/* session please token message format */
	SES_EVENT	event;		/* message type */
	SES_TK_DOMAIN	tk_req;	/* tokens requested */
} SES_MSG_TOKPL;


typedef struct {	/* session control give service message format */
	SES_EVENT	event;		/* message type */
} SES_MSG_CNTLGV;


typedef struct {	/* session minor/major sync point message format */
	SES_EVENT	event;		/* message type */
	unsigned	sync_type;	/* 1 => explicit conf NOT required
					   (only for minor sync REQ/IND) */
	SES_SPSN	spsn;		/* Sync point serial number */
} SES_MSG_SYNCP;


typedef struct {	/* session minor/major sync point message format */
	SES_EVENT	event;		/* message type 		*/
	SES_RSYNC_TYPE	rtype;		/* Resynchronize type 		*/
	SES_SPSN	spsn;		/* Sync point serial number 	*/
	SES_TOK_ASSIGN	tk_assign;	/* Token assignment after resync*/
} SES_MSG_RESYNC;


typedef struct {	/* session provider exception report message format */
	SES_EVENT	event;		/* message type */
	SES_PER_REASON	reason;		/* abort reason */
} SES_MSG_PER;


typedef struct {	/* session user exception report message format */
	SES_EVENT	event;		/* message type 		*/
	SES_UER_REASON	reason;		/* reason 			*/
} SES_MSG_UER;


typedef struct {	/* session activity start service message format */
	SES_EVENT	event;		/* message type 		*/
	SES_ACT_ID	act_id;		/* Activity Identifier		*/
} SES_MSG_ACTS;


typedef struct {	/* session activity resume service message format */
	SES_EVENT	event;		/* message type 		*/
	SES_ACT_ID	act_id;		/* Activity Identifier		*/
	SES_ACT_ID	old_act_id;	/* Old Activity Identifier	*/
	SES_SPSN	spsn;		/* Sync point serial number	*/
	struct {			/* old session connection id	*/
		SES_REF_ID	cg_cnref;	/* calling user conn ref*/
		SES_REF_ID	cd_cnref;	/* called user conn ref	*/
		SES_REF_ID	comm_cnref;	/* common  ref		*/
		SES_ADD_REF	add_ref;	/* additional ref info	*/
	}		old_conn_id;
} SES_MSG_ACTR;


typedef SES_MSG_UER SES_MSG_ACTI; /* session activity interrupt */

typedef SES_MSG_UER SES_MSG_ACTD; /* session activity discard */


typedef struct {	/* session activity end message format */
	SES_EVENT	event;		/* message type 		*/
	SES_SPSN	spsn;		/* sync point serial number
					   (unused in RSP & CNF)	*/
} SES_MSG_ACTE;


typedef struct {	/* release service message format */
	SES_EVENT	event;		/* message type */
	unsigned	accepted;	/* realease accepted */
} SES_MSG_RELEASE;


typedef struct {	/* session user abort service message format */
	SES_EVENT	event;		/* message type */
} SES_MSG_UABORT;


typedef struct {	/* session provider abort service message format */
	SES_EVENT	event;		/* message type */
	SES_PA_REASON	reason;		/* abort reason */
} SES_MSG_PABORT;



typedef union {		/* union of all session messages */
	SES_MSG		generic;
	SES_MSG_CONNECT	connect;
	SES_MSG_DATA	data;
	SES_MSG_XDATA	xdata;
	SES_MSG_TDATA	tdata;
	SES_MSG_CDATA	cdata;
	SES_MSG_TOKGV	tokgv;
	SES_MSG_TOKPL	tokpl;
	SES_MSG_CNTLGV	cntlgv;
	SES_MSG_SYNCP	syncp;
	SES_MSG_RESYNC	resync;
	SES_MSG_PER	per;
	SES_MSG_UER	uer;
	SES_MSG_ACTS	acts;
	SES_MSG_ACTR	actr;
	SES_MSG_ACTI	acti;
	SES_MSG_ACTD	actd;
	SES_MSG_ACTE	acte;
	SES_MSG_RELEASE	release;
	SES_MSG_UABORT	uabort;
	SES_MSG_PABORT	pabort;
} SES_MSG_BUFFER;


/*
 * Message casts for use in ses_action.c
 */
#define SU_CON(p)		((SES_MSG_CONNECT *)(p))
#define SU_REL(p)		((SES_MSG_RELEASE *)(p))
#define SU_DAT(p)		((SES_MSG_DATA *)(p))
#define SU_XDT(p)		((SES_MSG_XDATA *)(p))
#define SU_UAB(p)		((SES_MSG_UABORT *)(p))
#define SU_PAB(p)		((SES_MSG_PABORT *)(p))
#define SU_TDT(p)		((SES_MSG_TDATA *)(p))
#define SU_CDT(p)		((SES_MSG_CDATA *)(p))
#define SU_TKG(p)		((SES_MSG_TOKGV *)(p))
#define SU_TKP(p)		((SES_MSG_TOKPL *)(p))
#define SU_CTG(p)		((SES_MSG_CNTLGV *)(p))
#define SU_SYN(p)		((SES_MSG_SYNCP *)(p))
#define SU_RSY(p)		((SES_MSG_RESYNC *)(p))
#define SU_PER(p)		((SES_MSG_PER *)(p))
#define SU_UER(p)		((SES_MSG_UER *)(p))
#define SU_AS(p)		((SES_MSG_ACTS *)(p))
#define SU_AR(p)		((SES_MSG_ACTR *)(p))
#define SU_AI(p)		((SES_MSG_ACTI *)(p))
#define SU_AD(p)		((SES_MSG_ACTD *)(p))
#define SU_AE(p)		((SES_MSG_ACTE *)(p))
#define	TP_CON(p)		((TP_MSG_CONNECT *)(p))
#define	TP_DAT(p)		((TP_MSG_DATA *)(p))
#define	TP_XDT(p)		((TP_MSG_X_DATA *)(p))
#define	TP_DIS(p)		((TP_MSG_DISCONNECT *)(p))
#define SP(p)			((SPDU *)(p))
